var g_dBroadcastLatencyEstimate = 13.0;

var g_cPaneDefaultTopPadding = 10;
var g_cPaneContentDividerWidth = 1;

var g_cInfoPanePadding = 10;

var g_cEventViewerMinHeight = 20;

var g_cNotesInputHeight = 94;
var g_cNotesChooseUserHeight = 29;
var g_cNotesChooseChannelHeight = 23;

var g_cSearchChromeHeight = 78;

var g_cQuestionEntryHeight = 30;

// tab control that hosts multiple event viewers
function EventTabViewer(el, pViewer)
{
    var m_el = el;
    var m_pViewer = pViewer;
    
    var m_pLastStatus = null;
    var m_pSelectedView = null;
    
    // create our tab control (boolean specifies small tabs)
    var m_pTabControl = new TabControl(document.getElementById( "eventViewerBar" ), OnSelectView, false);
    
    // create the info tab
    var m_pInfo = new InfoTab(document.getElementById("infoDiv"), m_pViewer);
    m_pInfo.Title = "Info";
    m_pInfo.ViewID = "Info";
    m_pInfo.SetVisible(false);
    
    // create our table of contents
    var m_pContents = new TocViewer(document.getElementById("tocDiv"), m_pViewer);
    m_pContents.Title = "Contents";
    m_pContents.ViewID = "Contents";
    m_pContents.SetVisible(false);
    
    // create our transcript viewer
    var m_pTranscript = new TocViewer(document.getElementById("transDiv"), m_pViewer);
    m_pTranscript.Title = "Captions";
    m_pTranscript.ViewID = "Captions";
    m_pTranscript.SetVisible(false);
    
    this.RenderContents = function(arrEvents, arrTranscriptEvents)
    {
        // initialize our tab control with our array of viewers

        // Contents
        if (arrEvents.length > 0)
        {
            m_pTabControl.AddView(m_pContents);
            m_pContents.RenderContents(arrEvents);
        }

        // Transcript
        if (arrTranscriptEvents.length > 0)
        {
            m_pTabControl.AddView(m_pTranscript);
            m_pTranscript.RenderContents(arrTranscriptEvents);
        }

        // Info
        m_pTabControl.AddView(m_pInfo);

        m_pTabControl.UpdateCurrentView();
    }

    this.UpdateStatus = function UpdateStatus(status)
    {
        m_pLastStatus = status;
        m_pSelectedView.UpdateStatus(status);
    }

    // callback for clicking on tabs
    function OnSelectView(pView)
    {
        // If we're selecting a new tab, hide the old one
        if (m_pSelectedView && (m_pSelectedView != pView))
        {
            m_pSelectedView.SetVisible(false);
        }

        m_pSelectedView = pView;

        if (m_pLastStatus)
        {
            m_pSelectedView.UpdateStatus(m_pLastStatus);
        }
        
        m_pSelectedView.SetVisible(true);
        m_pSelectedView.SetFocus();
    }

    this.SetHeight = function SetHeight(controlHeight)
    {
        var eventViewerHeight = controlHeight - g_dContainerSpacing;
        m_el.style.height = controlHeight - g_dContainerSpacing + "px";

        // Client panes get eventViewerHeight minus tab bar height
        m_pInfo.OnResize(eventViewerHeight - g_dTabBarOffsetHeight);
        m_pContents.OnResize(eventViewerHeight - g_dTabBarOffsetHeight);
        m_pTranscript.OnResize(eventViewerHeight - g_dTabBarOffsetHeight);
    }
    
    this.SetWidth = function SetWidth(controlWidth)
    {
        m_el.style.width = controlWidth + "px";
    }
}

// Info tab (presenter name, bio, etc.)
function InfoTab(el, pViewer)
{
    var m_el = el;
    var m_elInfoContents = document.getElementById("infoContents");
    var m_elSpeakerBios = document.getElementById("speakerBios");

    var contributors = pViewer.contributors;

    for (var i = 0; i < contributors.length;  i++)
    {
        var displayName = contributors[i].DisplayName;
        // BUGBUG: Convert consecutive spaces to &nbsp; ?
        var bio = contributors[i].Bio;

        CreateChildElement(m_elSpeakerBios, "HR");

        var table = CreateChildElement(m_elSpeakerBios, "TABLE", "info");
        var row = table.insertRow(-1);

        var labelCell = row.insertCell(-1);
        labelCell.className = "label";
        SetText(labelCell, "Presenter:");

        var speakerCell = row.insertCell(-1);
        SetText(speakerCell, displayName);

        var bioText = CreateChildElement(m_elSpeakerBios, "DIV", "contentText");
        SetTextWithNewlineTranslation(bioText, bio);
    }

    this.RenderContents = function() { }
    
    this.UpdateStatus = function(status) { }
    
    this.SetVisible = function(bVisible)
    {
        SetVisible(m_el, bVisible);
    }
    
    this.SetFocus = function() { }

    this.OnResize = function OnResize(controlHeight)
    {
        var paneHeight = controlHeight - g_cPaneDefaultTopPadding;
        m_el.style.height = paneHeight + "px";
        m_elInfoContents.style.height = paneHeight - (g_cInfoPanePadding * 2) - g_cPaneContentDividerWidth + "px";
    }
}

// visual representation for events - pass in the parent, event object, and options
// options:
//  DeleteCallback - when present a delete button is shown for the visual and when clicked
//                   calls the passed in function with the event
function EventVisual(elParent, pEvent, pViewer, pOptions)
{
    var m_pEvent = pEvent;
    
    var m_pViewer = pViewer;
    
    var m_bEditing = false;
    var m_bSelected = false;
    var m_bHovering = false;

    var m_elContainer = CreateChildElement(elParent, "div", "eventVisual");
    var m_elActionPanel = CreateChildElement(m_elContainer, "div", "noteActionPanel");

    var innerContainer = CreateChildElement(m_elContainer, "div", "");
    innerContainer.style.position = "relative";

    var divTime = CreateChildElement(innerContainer, "div", "eventVisualTime");
    if (m_pViewer.isLive)
    {
        divTime.innerHTML = FormatFileTime(m_pEvent.Time);
    }
    else
    {
        divTime.innerHTML = FormatRelativeTime(m_pEvent.Time);
    } 

    if (pOptions && pOptions.ShowUsernames)
    {
        var divUser = CreateChildElement(innerContainer, "div", "eventVisualUser");
        if (m_pEvent.UserName)
        {
            divUser.innerHTML = "[" + m_pEvent.UserName + "]";
        }
        else
        {
            var spanAnonymousUser = CreateChildElement(divUser, "span", "eventVisualAnonymousUser");
            spanAnonymousUser.innerHTML = "[anonymous]"
        }
    }

    var divCaption = CreateChildElement(innerContainer, "div", "eventVisualElement");
    if (!m_pEvent.Caption)
    {
        divCaption.innerHTML = "";
    }
    else
    {
        // Create text node to escape HTML in text entry
        SetTextWithLinks(divCaption, m_pEvent.Caption);
    }

    // if we have a delete callback then create edit and delete buttons
    var m_elEditLink = null;
    var m_elCancelEditLink = null;
    var m_elDeleteLink = null;
    if (pOptions && (!m_pEvent.UserName || (m_pEvent.UserName == m_pViewer.userName)))
    {
        if (pOptions.DeleteCallback)
        {
            m_elEditLink = CreateChildElement(m_elActionPanel, "a", "editLink");
            m_elEditLink.href = "#";
            m_elEditLink.innerHTML = "edit";
            m_elEditLink.onclick = new Clicker(m_elEditLink, pOptions.EditCallback, pEvent);

            m_elCancelEditLink = CreateChildElement(m_elActionPanel, "a", "cancelEditLink");
            m_elCancelEditLink.href = "#";
            m_elCancelEditLink.innerHTML = "cancel edit";
            m_elCancelEditLink.onclick = new Clicker(m_elCancelEditLink, pOptions.CancelEditCallback);

            m_elDeleteLink = CreateChildElement(m_elActionPanel, "a", "deleteLink");
            m_elDeleteLink.href = "#";
            m_elDeleteLink.innerHTML = "delete";
            m_elDeleteLink.onclick = new Clicker(m_elDeleteLink, pOptions.DeleteCallback, pEvent);
        }
    }

    m_elContainer.onmouseover = function(e)
    {
        e = GetEvent(e);

        if (m_elDeleteLink)
        {
            m_elActionPanel.style.visibility = "visible";
        }

        m_bHovering = true;
        SetStyle();
        return false;
    }

    m_elContainer.onmouseout = function(e)
    {
        e = GetEvent(e);

        if (m_elDeleteLink)
        {
            m_elActionPanel.style.visibility = "hidden";
        }

        m_bHovering = false;
        SetStyle();
        return false;
    }


    function SetStyle()
    {
        if (m_bEditing)
        {
            m_elContainer.className = "eventVisualEditing";

            // If note action panel is displayed, show "cancel edit" link.
            if (m_elCancelEditLink)
            {
                m_elCancelEditLink.style.display = "inline";
                m_elEditLink.style.display = "none";
                m_elDeleteLink.style.display = "none";

                // We're changing padding to compensate for border, so tweak alignment to maintain position
                m_elActionPanel.style.top = "1px";
                m_elActionPanel.style.right = "1px";
            }
        }
        else
        {
            if (m_bSelected)
            {
                m_elContainer.className = "eventVisualSelected";
            }
            else if (m_bHovering)
            {
                m_elContainer.className = "eventVisualHover";
            }
            else
            {
                m_elContainer.className = "eventVisual";
            }

            // If note action panel is displayed, show "edit" and "delete" links.
            if (m_elCancelEditLink)
            {
                m_elCancelEditLink.style.display = "none";
                m_elEditLink.style.display = "inline";
                m_elDeleteLink.style.display = "inline";

                // We're changing padding to compensate for border, so tweak alignment to maintain position
                m_elActionPanel.style.top = "2px";
                m_elActionPanel.style.right = "2px";
            }
        }
    }

    this.SetSelected = function(bSelected, bEditing)
    {
        m_bSelected = bSelected;
        m_bEditing = bEditing;
        SetStyle();
    }

    this.GetContainer = function()
    {
        return m_elContainer;
    }
}

// general viewer for events
function EventViewer(el, pViewer, pOptions)
{
    var m_el = el;
    var m_pViewer = pViewer;

    var m_divContents = CreateChildElement(m_el, "div");
    var m_divScrollPane = null;
    var m_divToggleThumbs = null;
    
    var m_pSelectedItem = null;
    var m_arrTimestamps = null;
    var m_arrVisuals = new Array();
    
    var m_bInitialized = false;

    this.RenderContents = function(arrTimestamps)
    {
        // this implies that we will only view notes sorted by time                   
        arrTimestamps.sort(function(a, b) { return a.Time - b.Time });

        m_divContents.innerHTML = "";
        m_arrTimestamps = arrTimestamps;

        // clear existing visuals
        m_arrVisuals.splice(0, m_arrVisuals.length);

        // if we don't have any elements, hide the list
        if (m_arrTimestamps.length == 0)
        {
            SetVisible(m_divContents, false);
            return;
        }
        SetVisible(m_divContents, true);

        // create event visuals for our timestamps
        for (var i = 0; i < arrTimestamps.length; i++)
        {
            var pEventVisual = new EventVisual(m_divContents, arrTimestamps[i], m_pViewer, pOptions);
            m_arrVisuals.push(pEventVisual);

            var container = pEventVisual.GetContainer();

            if (!m_pViewer.isLive)
            {
                function eventClick(pItem)
                {
                    m_pViewer.SetVideoPosition(pItem.Time);
                    
                    if(pItem.EventTargetType != "PowerPoint")
                    {
                        m_pViewer.SelectObjectView(pItem.StreamID);
                    }
                }

                container.onclick = Clicker(container, eventClick, arrTimestamps[i]);
            }
            else
            {
                //BUGBUG: Should use CSS
                container.style.cursor = "normal";
            }
        }
    }

    // Highlight event and scroll into view.  If editing, indicate with different styling.
    function SelectItem(pItemToSelect, bEditing)
    {
        if (pItemToSelect == m_pSelectedItem)
        {
            // Value of bEditing may have changed, so call SetSelected anyway
            m_pSelectedItem.SetSelected(true, bEditing);
            return;
        }

        if (m_pSelectedItem)
        {
            m_pSelectedItem.SetSelected(false);
        }

        m_pSelectedItem = pItemToSelect;
        ScrollIntoView(m_pSelectedItem);
        m_pSelectedItem.SetSelected(true, bEditing);
    }

    // Find event visual in parallel list of event.
    this.SelectEditEvent = function(pEditEvent)
    {
        for (var i = 0; i < m_arrTimestamps.length; i++)
        {
            var pEvent = m_arrTimestamps[i];
            if (pEvent.EventID == pEditEvent.EventID)
            {
                var eventVisual = m_arrVisuals[i];
                SelectItem(eventVisual, true);
            }
        }
    }

    this.UpdateStatus = function(status)
    {
        if (!m_arrTimestamps)
        {
            return;
        }

        var iItem = GetCurrentItem(m_arrTimestamps, status.Time);
        if (iItem < m_arrVisuals.length)
        {
            SelectItem(m_arrVisuals[iItem]);
        }
    }
    
    this.SetVisible = function(bVisible)
    {
        SetVisible(m_el, bVisible);
    }
    
    this.OnResize = function OnResize(controlHeight)
    {
        m_el.style.height = controlHeight + "px";
    }

    function ScrollIntoView( pVisual )
    {
        var visHeight = pVisual.GetContainer().offsetHeight;
        var visTop = pVisual.GetContainer().offsetTop;


        var scrollHeight = m_el.offsetHeight


        
        var scroll = Math.max(0, (scrollHeight - visHeight) / 2);
        m_el.scrollTop = visTop - scroll;

    }
}

// Table of Contents (slide & thumbnail events)
function TocViewer(el, pViewer)
{
    var m_el = el;
    var m_pViewer = pViewer;

    var m_pEventViewer = new EventViewer(CreateChildElement(el, "div", "tocContents"), m_pViewer);

    this.RenderContents = function(arrTimestamps)
    {
        m_pEventViewer.RenderContents(arrTimestamps);
    }

    this.UpdateStatus = function(status)
    {
        m_pEventViewer.UpdateStatus(status);
    }

    this.SetVisible = function(bVisible)
    {
        SetVisible(m_el, bVisible);
    }

    this.SetFocus = function() { }

    this.OnResize = function OnResize(controlHeight)
    {
        var paneHeight = controlHeight - g_cPaneDefaultTopPadding;
        m_el.style.height = paneHeight + "px";
        m_pEventViewer.OnResize(paneHeight - g_cPaneContentDividerWidth);
    }
}
